import ast
from click.testing import CliRunner
from unittest.mock import patch, Mock
from llm_together import Together
from llm.cli import cli

from llm import Prompt, get_model, Response
import os
import pytest
from typing import List, Tuple


@patch("llm_together.together")
def test_together_response(mock_together):
    mock_together.Complete.create.return_value = {
        "output": {
            "choices": [
                {
                    "text": "hello"
                }
            ]
        }
    }
    mock_together.Complete.create_streaming.return_value = ["hello"]

    prompt = Prompt("hello", "")

    model = Together( {"name": "models/text-bison-001", "config": {}})
    items = list(model.response(prompt))
    
    assert items == ["hello"]



import llm
import together
from pydantic import Field
from typing import Optional

@llm.hookimpl
def register_models(register):

    together_instance = Together()
    model_list = together_instance.get_models()

    for model in model_list:
        if 'isFeaturedModel' in model and model['isFeaturedModel']:
            register(Together(model)) 

def not_nulls(data) -> dict:
    return {key: value for key, value in data if value is not None}

class Together(llm.Model):
    model_id = "llm-together"
    needs_key = "together"
    key_env_var = "TOGETHER_API_KEY"
    default_stop = "<human>"
    can_stream = True

    def get_models(self):
        together.api_key = self.get_key()
        return together.Models.list()
    
    def __init__(self, model=None):
        together.api_key = self.get_key()

        if (model is not None):
            self.model_id = model["name"]
            self.model = model

    class Options(llm.Options):
        temperature: Optional[float] = Field(
            description=(
                "What sampling temperature to use, between 0 and 2. Higher values like "
                "0.8 will make the output more random, while lower values like 0.2 will "
                "make it more focused and deterministic."
            ),
            ge=0,
            le=2,
            default=None,
        )
        max_tokens: Optional[int] = Field(
            description="Maximum number of tokens to generate.", default=256
        )
        top_p: Optional[float] = Field(
            description=(
                "An alternative to sampling with temperature, called nucleus sampling, "
                "where the model considers the results of the tokens with top_p "
                "probability mass. So 0.1 means only the tokens comprising the top "
                "10% probability mass are considered. Recommended to use top_p or "
                "temperature but not both."
            ),
            ge=0,
            le=1,
            default=None,
        )
        repetition_penalty: Optional[float] = Field(
            description=(
                "A number that controls the diversity of generated text by "
                "reducing the likelihood of repeated sequences. Higher values "
                "decrease repetition."
            ),
            ge=-2,
            le=2,
            default=None,
        )

    def execute(self, prompt, stream, response, conversation):
        kwargs = dict(not_nulls(prompt.options))

        user_prompt = "{}\n\n{}".format(prompt.system or "", prompt.prompt)
        history = ""
        stop = self.default_stop

        if 'config' in self.model:
            if conversation is not None:
                for message in conversation.responses:
                    if 'prompt_format' in self.model["config"] and self.model["config"]['prompt_format']:
                        history += self.model["config"]["prompt_format"].format(prompt = message.prompt) + " " + message.text() + "\n"
                    else:
                        history += "{}\n\n{}".format(message.prompt, message.text())+ "\n"

            if 'prompt_format' in self.model["config"] and self.model["config"]['prompt_format']:
                user_prompt = self.model["config"]["prompt_format"].format(prompt = user_prompt)


            if 'stop' in self.model["config"]:
                stop = self.model["config"]["stop"]

        if stream:
            for token in together.Complete.create_streaming(
                prompt =  history + "\n" + user_prompt,
                model = self.model_id,
                stop = stop,
                **kwargs,
            ):
                yield token

        else:
            output = together.Complete.create(
                prompt =  history + "\n" + user_prompt,
                model = self.model_id,
                stop = stop,
                **kwargs,
            )

            return [output['output']['choices'][0]['text']]



from setuptools import setup
import os

VERSION = "0.3"


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="llm-together",
    description="Plugin for LLM adding support for Together",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Kévin Quesada",
    url="https://github.com/wearedevx/llm-together",
    project_urls={
        "Issues": "https://github.com/wearedevx/llm-together/issues",
        "CI": "https://github.com/wearedevx/llm-together/actions",
        "Changelog": "https://github.com/wearedevx/llm-together/releases",
    },
    license="MIT",
    classifiers=["Apache-2.0"],
    version=VERSION,
    entry_points={"llm": ["together = llm_together"]},
    install_requires=["llm>=0.5", "together"],
    extras_require={"test": ["pytest"]}
)
